PIDIter += 1;

// Control variable: position from phaseStructureProperties
if (mesh.foundObject<uniformDimensionedVectorField>(CGName))
{
    CG = mesh.lookupObjectRef<uniformDimensionedVectorField>(CGName);
}
else
{
    FatalError  << "Registered object not found: " << CGName
                << exit(FatalError);
}

// Moving average of control variable over n last time steps (= PIDIters)
if (nCGList > 1)
{
    // Update list
    for (label i = 0; i < nCGList - 1; ++ i)
    {
        CGs[i] = CGs[i+1];
    }
    CGs[nCGList-1] = CG.value();

    // Calculate list mean value
    CG.value() = vector(0, 0, 0);
    for (label i = 0; i < nCGList; ++ i)
    {
        CG.value() += CGs[i];
    }
    CG.value() /= min(nCGList, PIDIter);
}

// Deviation
const dimensionedVector deviation(CG - CGTgt);
Info<< "  Deviation\t= " << deviation.value() << nl
    << "  ------------------------------------------"
    << "--------------------------------------------" << endl;

const dimensionedScalar& devX(deviation.component(vector::X));
const dimensionedScalar& devY(deviation.component(vector::Y));
const dimensionedScalar& devZ(deviation.component(vector::Z));

// PID controller
// P
const dimensionedVector uP
(
    "uP",
    (
        X*devX*inletKP.component(vector::X)
      + Y*devY*inletKP.component(vector::Y)
      + Z*devZ*inletKP.component(vector::Z)
    )
);
Info<< "  uP\t\t= " << uP.value() << endl;

// I
uISum +=
    runTime.deltaT()
   *(
        X*devX*inletKI.component(vector::X)
      + Y*devY*inletKI.component(vector::Y)
      + Z*devZ*inletKI.component(vector::Z)
    );
const dimensionedVector& uI = uISum;
Info<< "  uI\t\t= " << uI.value() << endl;

// D
dimensionedVector dDeviation(deviation - deviationOld);
if (PIDIter <= nCGList)
{
    dDeviation.value() =
        (CGs[nCGList-1] - CGs[nCGList-PIDIter])/scalar(PIDIter);
}

const dimensionedScalar& dDevX(dDeviation.component(vector::X));
const dimensionedScalar& dDevY(dDeviation.component(vector::Y));
const dimensionedScalar& dDevZ(dDeviation.component(vector::Z));

const dimensionedVector uD
(
    "uD",
    (
        X*dDevX*inletKD.component(vector::X)
      + Y*dDevY*inletKD.component(vector::Y)
      + Z*dDevZ*inletKD.component(vector::Z)
    )
   /runTime.deltaT()
);
deviationOld = deviation;
Info<< "  uD\t\t= " << uD.value() << nl
    << "  ------------------------------------------"
    << "--------------------------------------------" << endl;

// Correcting variable
dimensionedVector uPID("uPID", uP + uI + uD);
Pstream::scatter(uPID);
Info<< "  uPID\t\t= " << uPID.value() << nl << endl;
