# Description

Tutorial demonstrates a setup for simulating stratified flows with
OpenFOAM Hybrid and is based on Fabre et al. (1987)[^1] channel experiment
run 250. The case represents a stratified co-curent flow
of air and water with a relatively smooth interface.

| Case                                 | 250              |
| --------                             | --------         |
| Liquid flow rate [l/s]               | 3                |
| Gas flow rate [l/s]                  | 45.4             |
| Mean liquid level [mm]               | 38               |
| Pressure gradient [Pa/m]             | 2.1              |
| Liquid superficial velocity [m/s]    | 0.15             |
| Gas superficial velocity [m/s]       | 2.27             |
| Liquid inlet velocity [m/s]          | 0.394737         |
| Gas inlet velocity [m/s]             | 3.66129          |

# Technical Information

* 2D domain
* height = 0.1 m, length = 12 m, (width = 0.2 m, one cell)
* adjusted g components to account for 0.1% slope in the experiment
  - angle = arctan(slope/100)
  - gx = g*sin(angle)
  - gy = -g*cos(angle)
* channel split in two blocks:
  - water: height 0.038 m, 8 cells in y direction (height)
  - air: height 0.062 m, 12 cells in y direction (3:2 ratio)
  - (note: dy in air is ~10% larger)
  - 144 cells in x direction (length)
  - 2880 total mesh cells
* two inlets (air, water), two outlets
  - velocity ramp is used for initial 5 s to soften the transient
* simulated time 30 s, about a flow through time for water
* StrubeljTiselj drag formulation
* partial elimination
* interface compression
* turbulence damping with kOmegaSSTItEgorov (see Frederix et al., 2018)[^2]
  - No damping in water, damping in air with delta = 1e-4
* interface indicator method with gradAlpha

# Pre-Processing

* setFields: fills the channel with water up to y = 0.038 m

# Post-Processing

Function verticalLine is called at the end of run.

FunctionObjects used at run time:
* residuals (p_rgh)
* flowRatePatch (to optionally check both inlet flow rates)

Bash script:
* Gnuplot script createGraphs

Results are compared to experimental data of Fabre et al. (1987)[^1]. Find all
result files in the validation directory.

# Keywords

gentop, stratified flow, turbulence damping

# Additional Remarks

* Optional
  - add damping to the water phase
  - mesh refinement (current is very coarse)
  - switch from PISO -> PIMPLE run, use momentumPredictor
* Approximate serial run time is 200 s with the current setup

# Literature

[^1]: Fabre, J., Masbernat, L., & Suzanne, C. (1987).
Experimental data set no. 7: stratified flow, part 1: local structure.
Multiphase Science and Technology, 3(1-4), 285-301.

[^2]: Frederix, E.M.A., Mathur, A., Dovizio, D., Geurts, B.J., & Komen, E.M.J.
(2018). Reynolds-averaged modeling of turbulence damping near a large-scale
interface in two-phase flow. Nucl. Eng. Des., 333, 122-130.
